//
//  IZegoAvatarView.h
//  ZegoAvatar
//
//  Created by justinsong on 2021/12/3.
//
//
//  ZegoAvatarView.h
//  virtual
//
//  Created by justinsong on 2021/11/17.
//

#pragma once
#import <UIKit/UIKit.h>
#import "ZegoTypes.h"
#import "IZegoCharacter.h"
#import "ZegoVector3.h"

NS_ASSUME_NONNULL_BEGIN

#pragma mark - IZegoAvatarView

@protocol IZegoAvatarView <NSObject>

@required

/// 设置 View 背景图片
@property (nonatomic, strong) UIImage *backgroundImage;


/// 释放 view 的相关资源
-(void)releaseResources;
/// 设置 View 的角色
/// 
/// @param character character
- (void)setCharacter:(id<IZegoCharacter>)character;

/// 设置 View 的角色
///
/// Description: callback 设置角色后，在绘制角色的第一帧完成时回调。
///
/// @param character character
/// @param callback  callback
- (void)setCharacter:(id<IZegoCharacter>)character callback:(ZegoAvatarCharacterCallback)callback;

/// 移除 View 的角色
///
/// @param character character
- (void)removeCharacter:(id<IZegoCharacter>)character;

/// 移除 View 的所有角色
- (void)removeAllCharacter;

/// 暂停 View 的渲染
- (void)pause;

/// 恢复 View 的渲染
- (void)resume;

/// 截取 View 视图当前帧图片
///
/// @return View 视图当前帧图片
- (UIImage *)captureScreenshot;

/// 录制 View 的视频
///
/// Description:
///   recordConfig.videoPath 路径下有同名文件会被覆盖。
///   onStartRecordCallback 在开始录制时回调，只回调一次。
///
/// @param recordConfig 录制参数配置
/// @param onStartRecordCallback callback
- (BOOL)startRecord:(ZegoRecordConfig *)recordConfig onStartRecordCallback:(void (^)(NSInteger errorCode, NSString *info))onStartRecordCallback;

/// 结束 View 的录制
///
/// Description: onStopRecordCallback 在结束录制后回调。
///
/// @param onStopRecordCallback callback
- (void)stopRecord:(void (^)(void))onStopRecordCallback;

/// 暂停录制
///
/// Description: 录制视频中暂停和恢复, 会带来很多不可预见的兼容性问题, 未来版本将废弃该接口。
- (void)pauseRecord;

/// 恢复录制
///
/// Description: 录制视频中暂停和恢复, 会带来很多不可预见的兼容性问题, 未来版本将废弃该接口。
- (void)resumeRecord;

/// 展示 View 中角色的影子
///
/// Description: 默认不展示。
///
/// @param show show
- (void)showShadow:(BOOL)show;

/// 是否能缩放角色
///
/// Description: 默认开启。
///
/// @param enable enable
- (void)enableCameraScale:(BOOL)enable;

/// 是否能旋转角色，默认开启
///
/// Description: 默认开启。
///
/// @param enable enable
- (void)enableCameraRotate:(BOOL)enable;

/// 设置相机机位
///
/// @param cameraName 机位描述
- (void)setCamera:(NSString *)cameraName;

/// 播放多人动画
///
/// Description:
///   将角色动画的绝对路径作为 value ，角色 roleID 作为 key ，放入 map 中。
///   callback 在动画完成后回调。
///
/// @param map map
/// @param callback callback
- (void)playMultiCharacterAnimation:(NSDictionary<NSString *, NSString *> *)map
                           callback:(ZegoAvatarMultiCharacterAnimationCallback)callback;

- (void)playMultiCharacterAnimation:(NSDictionary<NSString *, NSString *> *)map
                               loop:(BOOL)loop speed:(float)speed blend:(BOOL)blend callback:(ZegoAvatarMultiCharacterAnimationCallback)callback;

/// 设置 AvatarView 的纹理监听回调
///
/// Description: 回调 AvatarView 的纹理ID。
///
/// @param callback callback
- (void)setTextureUpdateCallback:(ZegoAvatarViewTextureUpdateCallback _Nullable)callback;


/**
 * 移除纹理更新回调通知
 */
- (void)removeTextureUpdateListener;


/**
 * 以自定义配置设置机位
 * @param config 机位配置 ZEGO 提供
 */
- (void)setCameraByConfig:(NSString *)config;

/**
 * 设置相机的旋转模式
 * @param mode 旋转模式。0 ：Avatar    1 : Camera
 */
- (void)setCameraRotateMode:(ZegoAvatarCameraRotateMode)mode;

/**
 * 设置相机关注角色
 * @param character 角色对象
 */
- (void)setCameraCharacter:(id<IZegoCharacter>)character;

/**
 * 执行一个相机运动
 * @param motionConfig 相机配置
 */
- (void)playCameraMotion:(NSString *)motionConfig callback:(ZegoAvatarCameraMotionCallback)callback;
/**
 * 执行一个相机运动
 * @param motionConfig 相机配置
 */
- (void)playCameraMotion:(NSString *)motionConfig alignPoaition:(ZegoVector3*)position
                callback:(ZegoAvatarCameraMotionCallback)callback;
/**
 * 执行一个相机运动
 * @param motionConfig 相机配置
 */
- (void)playCameraMotion:(NSString *)motionConfig part:(ZegoAvatarPart)part follow:(bool)follow callback:(ZegoAvatarCameraMotionCallback)callback;

/**
 * 停止相机运动
 */
- (void)stopCameraMotion;

/**
 * 设置相机过渡效果，0 表示关闭过渡效果
 */
-(void)setCameraSmooth:(float)smooth;

/// 设置场景
/// Description: 设置当前view场景。
///
/// @param path 素材路径
/// @param callback callback
- (void)setScene:(NSString *)path
                           callback:(ZegoAvatarSetSceneCallback)callback;

/// 移除场景
///
/// Description: 移除当前view已设置的场景。
- (void)removeScene;

/// 设置场景缩放比
///
/// Description: 设置当前view已设置场景的缩放比。
///
/// @param x  x轴方向
/// @param y  y轴方向
/// @param z  z轴方向
- (void)setSceneScale:(float)x
                    y:(float)y
                    z:(float)z;

/// 开始录制 avatar 的gif图片
///
/// Description: 开始录制。
- (void)startRecordFramePack;

/// 停止录制 avatar 的gif图片
///
/// Description: 停止录制。
///   callback 在录制完成后回调。
///
/// @param callback callback
- (void)stopRecordFramePack:(ZegoAvatarRecordFramePackCallback)callback;

/// 录制 avatar 的 gif 图片且自动停止
///
/// Description: 开始录制并且录制 recordDuration 时长后自动停止。
///   callback 在录制完成后回调。
///
/// @param duration 录制时长，单位是 ms
/// @param callback callback
- (void)startRecordFramePackByDuration:(int)duration
                              callback:(ZegoAvatarRecordFramePackCallback)callback;

/// 录制 avatar 执行动画的 gif 图片
///
/// Description: avatar 按照 animationJson 配置的动画录制 gif 图片。
///   singleCallback 在单个动画录制完成后回调。
///   callback 在所有动画录制完成后回调。
///
/// @param animationJson 动画配置
/// @param singleCallback singleCallback
/// @param callback callback
- (void)recordFramePackByAnimations:(NSString *)animationJson
               singleRecordCallback:(ZegoAvatarRecordSingleAnimationFramePackCallback)singleCallback
                  allRecordCallback:(ZegoAvatarRecordAnimationsFramePackCallback)callback;


- (void)playEffect:(NSString *)effectJson
    effectCallback:(ZegoAvatarIntStringCallback)effectCallback;

/**
 * 播放场景特效
 */
- (void)playEffect:(NSString *)effectJson
              loop:(bool)loop
    effectCallback:(ZegoAvatarIntStringCallback)effectCallback;

/**
 * 移除指定的场景特效
 */
- (void)removeEffect:(NSString *)effectID;




@end

NS_ASSUME_NONNULL_END
