//
//  ZegoAvatarService.h
//  ZegoAvatar
//
//  Created by lihua.wei on 2021/12/1.
//

#pragma once

#import <Foundation/Foundation.h>
#import "ZegoTypes.h"
#import "IZegoCharacter.h"
#import "IZegoAvatarView.h"
#import "IZegoInteractEngine.h"
#import "IZegoAvatarCaptureProvider.h"
#import "IZegoFaceMaskCaptureProvider.h"

NS_ASSUME_NONNULL_BEGIN

#pragma mark - ZegoAvatarServiceDelegate

/// ZegoAvatarService 回调代理
@protocol ZegoAvatarServiceDelegate <NSObject>

@optional

/// ZegoAvatarService 错误回调
///
/// @param code 错误码
/// @param desc 错误描述
- (void)onError:(ZegoAvatarErrorCode)code desc:(NSString *)desc;

/// ZegoAvatarService 初始化状态回调
///
/// @param state ZegoAvatarService 初始化状态
- (void)onStateChange:(ZegoAvatarServiceState)state;

@end

#pragma mark - ZegoAvatarService

@interface ZegoAvatarService : NSObject

/// 获取 ZegoAvatarService 单例对象
///
/// @return ZegoAvatarService 单例对象
+ (instancetype)sharedInstance;

/// ZegoAvatarService 初始化方法
///
/// Description: Use initWithConfigV2 instead.
///
/// @param config 配置参数
- (void)initWithConfig:(ZegoServiceConfig *)config __deprecated __deprecated_msg("use initWithConfigV2 instead.");

/// ZegoAvatarService 初始化方法V2
///
/// @param config 配置参数
- (void)initWithConfigV2:(ZegoServiceConfigV2 *)config;

/// 设置 ZegoAvatarService 监听
///
/// @param observer observer
- (void)addServiceObserver:(id<ZegoAvatarServiceDelegate>)observer;

/// 移除 ZegoAvatarService 监听
///
/// @param observer observer
- (void)removeServiceObserver:(id<ZegoAvatarServiceDelegate>)observer;

/// 获取 ZegoAvatarService 初始化状态
///
/// @return ZegoAvatarService 初始化状态
- (ZegoAvatarServiceState)getState;

/// 创建 Avatar 视图对象
///
/// @param frame frame
- (nullable UIView<IZegoAvatarView> *)createAvatarView:(CGRect)frame;

/// 创建 Avatar 角色对象
///
/// @param modelPath 角色资源路径
- (nullable id<IZegoCharacter>)createCharacter:(NSString *)modelPath;

/// 创建捕获 Avatar 帧的对象
///
/// @return 捕获 Avatar 帧的对象
- (id<IZegoAvatarCaptureProvider>)createCaptureProvider;

/// 创建捕获 Avatar AR 帧的对象
///
/// @return 捕获 Avatar AR 帧的对象
- (id<IZegoFaceMaskCaptureProvider>)createFaceMaskCaptureProvider;

/// 获取互动引擎，ZegoAvatarService 初始化后才能获取
///
/// @return 互动引擎
- (nullable id<IZegoInteractEngine>)getInteractEngine;

/// 获取当前 SDK 版本号
///
/// @return 当前 SDK 版本号
+ (NSString *)getVersion;

/// 暂停所有 SDK 相关工作
- (void)pause;

/// 恢复所有 SDK 相关工作
- (void)resume;

/// 修改 SDK 内部朝向
///
/// @param orientation orientation
- (void)refreshOrientation:(ZegoAvatarPreferOrientation)orientation;

@end

NS_ASSUME_NONNULL_END
