//
//  IZegoCharacter.h
//  ZegoAvatar
//
//  Created by justinsong on 2021/12/3.
//


#pragma once
#import <Foundation/Foundation.h>
#import <CoreGraphics/CoreGraphics.h>
#import "ZegoTypes.h"
#import "ZegoVector3.h"

NS_ASSUME_NONNULL_BEGIN

#pragma mark - IZegoCharacter

@protocol IZegoCharacter <NSObject>

@required

/// 设置角色表情数据
///
/// Description:  表情数据可以由 AI 引擎 [startDetectExpression:] 获取。
///
/// @param exp 表情数据
- (void)setExpression:(ZegoExpression *)exp;

/// 设置肢体驱动系数
///
/// Description:  肢体驱动系数可以由 AI 引擎 [startDetectPose:] 获取。
///
/// @param poseInfo 肢体驱动系数
- (void)setPose:(unsigned long)poseInfo;

/// 设置角色素材资源包
///
/// @param packagePath 资源包绝对路径
- (void)setPackage:(NSString *)packagePath;

/// 设置角色素材资源包
///
/// Description: callback 在设置资源完成时回调。
///
/// @param packagePath 资源包绝对路径
/// @param callback callback
- (void)setPackage:(NSString *)packagePath callback:(_Nullable ZegoAvatarSetPackageCallback)callback;

/// 设置角色素材资源包
///
/// Description: callback 在设置资源完成时回调。
///
/// @param packagePath 资源包绝对路径
/// @param repel 资源是否互斥
/// @param callback callback
- (void)setPackage:(NSString *)packagePath
             repel:(BOOL)repel
          callback:(_Nullable ZegoAvatarSetPackageCallback)callback;

/// 播放角色动画
///
/// @param animation 动画资源绝对路径
- (void)playAnimation:(NSString *)animationPath;

/// 播放角色动画
///
/// Description: 动画资源展示完成后回调。
///
/// @param animation  动画资源绝对路径
/// @param callback callback
- (void)playAnimation:(NSString *)animation callback:(_Nullable ZegoAvatarAnimationCallback)callback;

- (void)playAnimation:(NSString *)animation loop:(BOOL)loop speed:(float)speed blend:(BOOL)blend callback:(_Nullable ZegoAvatarAnimationCallback)callback;

/// 角色动画开关，是否可以播放动画
///
/// Description:  YES 可以播放动画 ，NO 不可以播放动画，默认 YES 。
///
/// @param enableAnimation enable or not
- (void)enableAnimation:(BOOL)enableAnimation;

/// 角色舌头动画开关，是否可以播放舌头动画
///
/// Description: YES 启用舌头动画 ，NO 关闭舌头动画，默认 YES 。
///
/// @param enableAnimation  enable or not
- (void)enableTongueAnimation:(BOOL)enableAnimation;

/// 获取模型路径
///
/// @return 模型路径
- (NSString *)getModelPath;

/// 获取模型ID
///
/// @return 模型ID
- (NSString *)getRoleID;

/// 设置角色单个维度形变
///
/// @param partKey 形变维度
/// @param value 形变数值
- (void)setDeformation:(NSString *)partKey value:(float)value;

/// 获取角色单个维度形变值
///
/// @param partKey 形变维度
/// @return 形变数值
- (CGFloat)getDeformation:(NSString *)partKey;

/// 设置角色多个维度形变
///
/// Description: 将角色多个形变维度组合成 json 传入，格式参考 [getDeformation]。
///
/// @param data 形变数值
- (void)setDeformation:(NSString *)data;

/// 获取角色全部维度形变数据
///
/// @return 角色全部维度形变数据
- (NSString *)getDeformation;

/// 设置角色装扮
///
/// Description: 将角色多个装扮信息组合成 json 传入，格式参考 [getMakeupConfig]。
///
/// @param config 装扮信息
- (void)setMakeupConfig:(NSString *)config;

/// 获取角色所有装扮信息
///
/// @return 角色所有装扮信息
- (NSString *)getMakeupConfig;

/// 移除角色装扮
///
/// @param makeupType 装扮类型
- (void)removePackage:(ZegoAvatarMakeupType)makeupType;

/// 设置角色皮肤颜色
///
/// Description: 支持 RGBA 色值和十六进制色值。
///
/// @param color 色值
- (void)setSkinColor:(NSString *)color;

/// 设置角色头发颜色  [支持渐变色]
///
/// Description:
///   支持 RGBA 色值和十六进制色值。
///   发色支持渐变色。
///
/// @param startColor 开始颜色
/// @param endColor 结束颜色
- (void)setHairColor:(NSString *)startColor endColor:(NSString *)endColor;

/// 设置角色嘴唇颜色
///
/// Description: 支持 RGBA 色值和十六进制色值。
///
/// @param color 色值
- (void)setLipColor:(NSString *)color;

/// 设置角色装扮改变的监听
///
/// Description: 装扮改变时回调。
///
/// @param callback callback
- (void)setMakeupUpdateCallback:(ZegoAvatarMakeupUpdateCallback)callback;

/**
 * 开启角色的头部追踪效果
 * 头部追踪：即模拟角色在静止状态装扮随风飘动的效果
 * @param enable  true: 开启  false: 关闭
 */
- (void)enableHeadTracking:(BOOL)enable;

/**
 * 开启布料的风动效果
 * 风动效果：即模拟角色在静止状态装扮随风飘动的效果
 * @param enable  true: 开启  false: 关闭
 */
- (void)enableBlow:(BOOL)enable;

/**
 * 设置角色位置
 *
 * @param position 三维坐标系的位置，在三位空间指定角色的位置，通常默认出生位置为(0,0,0), 1个单位为真实世界的100cm
 */
- (void)setPosition:(ZegoVector3 *)position;

/**
 * 设置角色旋转
 *
 * @param rotation 三维坐标系的旋转值，x表示绕x轴旋转指定角度，y表示绕y轴旋转指定角度，z表示绕z轴旋转指定角度 （顺时针旋转）
 */
- (void)setRotation:(ZegoVector3 *)rotation;

/**
 * 移除指定装扮
 *
 * @param packageID 装扮的IDName
 */
- (void)removePackageByID:(NSString *)packageID;


/// 设置角色挂饰资源包
///
/// Description: callback 在设置资源完成时回调。
///
/// @param packagePath 资源包绝对路径
/// @param callback callback
- (void)setAttachment:(NSString *)packagePath callback:(_Nullable ZMWAvatarSetAttachmentCallback)callback;

- (void)setAttachment:(NSString *)packagePath roleAnimationPath:(NSString *)roleAnimationPath callback:(_Nullable ZMWAvatarSetAttachmentCallback)callback;

/// 移除角色挂饰
///
/// @param packageName
- (void)removeAttachment:(NSString *)packageName;

/**
 * 移除角色所有挂饰
 */
- (void)clearAttachment;

/**
 * 获取角色所有挂饰ID
 */
- (NSArray<NSString *> *)getAllAttachment;


@end

NS_ASSUME_NONNULL_END

